'use strict'
var tape     = require('tape')
var level    = require('level-test')()
var sublevel = require('level-sublevel/bytewise')
var pull     = require('pull-stream')
var ssbKeys  = require('ssb-keys')
var createFeed = require('ssb-feed')

var hexpp    = require('hexpp')

module.exports = function (opts) {

  var codec = opts.codec

  var content = {
      "type":"post",
      "is":"text",
      "text":"test",
      "timezone":300,
      "rebroadcasts":{
        "msg":
          '1BHEHMwZlikXB3o1mg+fP3GVo/+Xb7p46u3rqt/hHkA=.blake2s',
        "feed":
          "rbU6CvdwBXxO/fDyoKuRyKxmZYyeb5+l87R9XVkN8bs=.blake2s",
        "timestamp":1414078805677,
        "timezone":300
      }
    }

  var msg = {
    author: opts.hash('TEST_AUTHOR'),
    previous: opts.hash('TEST_PREVIOUS'),
    timestamp: Date.now(),
    sequence: 10,
    content: content
  }

  var signed = {}

  for(var k in msg)
    signed[k] = msg[k]

  signed.signature = new Buffer(64).toString('base64')

  tape('Message', function (t) {
    var enc = codec.encode(msg)
    console.log('**** Message ****')
    console.log(hexpp(enc))
    var o = codec.decode(enc)
    t.deepEqual(o, msg)
    t.end()
  })

  tape('Signed', function (t) {
    var enc = codec.encode(signed)
    console.log('**** Signed ****')
    console.log(hexpp(enc))
    var o = codec.decode(enc)
    t.deepEqual(o, signed)
    t.end()
  })

  tape('known error case 1', function (t) {

    var db = sublevel(level('test-ssb-encoding', {
      valueEncoding: codec
    }))
    var ssb = require('../')(db, opts)
    var feed = createFeed(ssb, opts.generate(), opts)

    feed.add(content, function (err, msg) {
      if(err) throw err
      t.assert(!!msg)
      t.assert(!!msg.key)
      t.assert(!!msg.value)
      pull(
        ssb.createFeedStream(),
        pull.collect(function (err, ary) {
          if(err) throw err
          t.equal(ary.length, 1)
          console.log(ary)
          t.end()
        })
      )
    })

  })
}


if(!module.parent)
  module.exports(require('../defaults'))
