var pull = require('pull-stream')
var pushable = require('pull-pushable')

exports.name = 'replicate2'
exports.version = '1.0.0'
exports.manifest = { stream: 'duplex', clock: 'sync' }
exports.init = function (sbot) {

  var clock = {}, streams = []

  //messages sucessfully written to our database.
  function addLatest (upto) {
    clock[upto.id] = Math.max(clock[upto.id] || 0, upto.sequence)
  }

  pull(sbot.latest(), pull.drain(addLatest)) //from database

  sbot.post(function (msg) { //incremental
    addLatest({id: msg.value.author, sequence: msg.value.sequence})
    send(msg)
  })

  function send (msg) { //check if any streams want this message
    var id = msg.author, sequence = msg.sequence
    streams.forEach(function (stream) {
      if(stream.clock[id] = sequence - 1) {
        streams.source.push(msg)
        stream.clock[id] = sequence
      }
    })
  }

  function read (id, seq) {
    //TODO, handle messages received on this feed while reading.
    pull(
      sbot.createHistoryStream({id: id, sequence: seq}),
      pull.drain(send)
    )
  }

  return {
    stream: function (cb) {
      var duplex = {
        clock: {}
        source: pushable(),
        sink: pull.drain(function (msg) {
          if(msg.clock) {
            for(var id in msg.clock) {
              var sequence = msg.clock[id]
              //if this is a new request, send these messages.
              if(!duplex.clock[id] && sequence < clock[id])
                read(id, msg.clock[id])

              duplex.clock[id] = msg.clock[id])
            }
          }
        }, function (err) {
          //stream ended, remove from list
          streams.splice(streams.indexOf(duplex), 1)
        })
      }

      streams.push(duplex)
      duplex.push({clock: clock})
      return duplex
    },
    clock: function () {
      return clock
    }
  }

}
